# -*- coding: utf-8 -*-
"""
/***************************************************************************
 RTKubePositionDialog
                                 A QGIS plugin
 RTKube interface
                             -------------------
        begin                : 2018-07-17
        git sha              : $Format:%H$
        copyright            : (C) 2018 by Oslandia
        email                : infos@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from __future__ import print_function
from __future__ import absolute_import

import os
import re
import serial

from qgis.PyQt import QtWidgets, uic, QtCore, QtGui

from qgis.utils import iface
from qgis.gui import QgsMessageBar
from qgis.core import Qgis

from .rtkubeSerial import rtkubeSerial

from .constantes import *

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'rtkube_position_dialog_base.ui'))


class RTKubePositionDialog(QtWidgets.QDialog, FORM_CLASS):

    settings = QtCore.QSettings()
    
    def __init__(self, parent=None):
        """Constructor."""
        super(RTKubePositionDialog, self).__init__(parent)
        # Set up the user interface from Designer.
        # After setupUI you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        reCoordinates = QtCore.QRegExp("[-]?[0-9]{1,2}.[0-9]{1,9}$")
        valCoordinates = QtGui.QRegExpValidator(reCoordinates)
        self.lattitude.setValidator(valCoordinates)
        self.longitude.setValidator(valCoordinates)

        reAlt = QtCore.QRegExp("[-]?[0-9]{1,4}.[0-9][0-9]$")
        valAlt = QtGui.QRegExpValidator(reAlt)
        self.altitude.setValidator(valAlt)

        self.lattitude.textChanged.connect(self.validate)
        self.longitude.textChanged.connect(self.validate)
        self.altitude.textChanged.connect(self.validate)

        self.validate()

        self.isConnected = False
        self.isModeFixed = False

        self.connectButton.clicked.connect(self.connection)
        self.sendButton.clicked.connect(self.sendPosition)
        self.infoButton.clicked.connect(self.getInformation)

        self.setEnabledWidgets()

    def validate(self, t=None):
        valid = self.lattitude.hasAcceptableInput() and \
            self.longitude.hasAcceptableInput() and \
            self.altitude.hasAcceptableInput()
        self.sendButton.setEnabled(valid and self.isConnected)

    def connection(self):
        if self.isConnected == False:
            try:
                self.ser = serial.Serial(
                    self.settings.value('/GeoCube/serialportbase', ''), baudrate=BAUDRATE, timeout=TIMEOUT)
                self.isConnected = True
                self.getMode()
            except serial.serialutil.SerialException as e:
                self.errors(str(e))

        else:
            self.disconnection()

        self.changeButtonText()

    def disconnection(self):
        if self.isConnected:
            self.ser.close()
        self.isConnected = False

        self.lattitude.setText("")
        self.longitude.setText("")
        self.altitude.setText("")
        self.accuracy.setText("")

        self.changeButtonText()

    def changeButtonText(self):
        if self.isConnected:
            self.connectButton.setText(u"Disconnection")
        else:
            self.connectButton.setText(u"Connection")
        self.setEnabledWidgets()

    def errors(self, msg):
        iface.messageBar().pushMessage(
            "RTKube", msg, level=Qgis.Critical, duration=DURATION)
        self.disconnection()

    def setEnabledWidgets(self):
        self.sendButton.setEnabled(self.isConnected)
        self.infoButton.setEnabled(self.isConnected)

    def sendPosition(self):
        self.ser.flushInput()

        cmd = "gp 2 {LAT} {LON} {ALT} {ACC}\r\n".format(LAT=self.lattitude.text(), LON=self.longitude.text(),
                                                        ALT=self.altitude.text(), ACC=self.accuracy.text())

        self.ser.write(cmd.encode())

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != cmd.rstrip('\r\n'):
            self.criticalAlert("Command badly interpreted\n"+cmd)
            return

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret.find("Position fixed") == -1:
            self.criticalAlert("Position not fixed")
            return

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != "OK":
            self.criticalAlert("OK should be returned")
            return

        # flush
        ret = self.ser.readline()  # '>'
        ret = self.ser.readline()  # ''

        # Position is fixed, now write it
        cmd = "gG\r\n"
        self.ser.write(cmd.encode())

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != cmd.rstrip('\r\n'):
            self.criticalAlert("Command badly interpreted\n"+cmd)
            return

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != "OK":
            self.criticalAlert("OK should be returned")
            return

        # flush
        ret = self.ser.readline()  # '>'
        ret = self.ser.readline()  # ''

        QtWidgets.QMessageBox.information(
            self, "RTKube base information", "Position fixed")

    def getInformation(self):
        self.ser.flushInput()

        cmd = "cd GPS\r\n"
        self.ser.write(cmd.encode())

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != cmd.rstrip('\r\n'):
            self.criticalAlert("Command badly interpreted\n"+cmd)
            return

        ret = 'RTKube Information\n'
        txt = ''
        while ret.rstrip('\r\n') != 'OK':
            ret = self.ser.readline().decode('ascii')
            txt += ret.replace('\r', '')

        QtWidgets.QMessageBox.information(self, "RTKube base information", txt)

    def getMode(self):
        self.ser.flushInput()

        cmd = "gp\r\n"
        self.ser.write(cmd.encode())

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        if ret != cmd.rstrip('\r\n'):
            self.criticalAlert("Command badly interpreted\n"+cmd)
            return

        ret = self.ser.readline().decode('ascii').rstrip('\r\n')
        self.isModeFixed = ret == "Fixed mode"

        if self.isModeFixed:
            self.fsLabel.setText("Fixed")
            ret = self.ser.readline().decode('ascii')
            self.labelPosition(ret)
        else:
            self.fsLabel.setText("Surveying")

    def labelPosition(self, txt):
        # ['46.100000000', '2.400000000', '1230', '50000']
        args = re.findall('\\b\\d+\.?\d+\\b', txt)
        print(args)

        if len(args) != 4:
            self.criticalAlert("Error in parsing fixed position")
            return

        self.lattitude.setText(args[0])
        self.longitude.setText(args[1])
        self.altitude.setText(args[2])
        self.accuracy.setText(str(float(args[3])/10000))

    def criticalAlert(self, msg):
        QtWidgets.QMessageBox.critical(self, "RTKube base", msg)
